# cython: c_string_type=str, c_string_encoding=utf8
from QPT cimport QuestionPaper

cdef class QPTGenerator:
    """ QPTGenerator is a very fast Question Paper Generator

    :argument
        mark_of: Dict[str, List[int]]:
            - A dict of constraint_name and list of mark distributions
        question_no: List[int]:
            - A list of question no. associated with list of question-wise mark distributions.
            - Repetition of same question no. indicates subquestions of that question.
    :returns
        allot_by_question_no: Dict[str, List[int]]:
            - A dict of constraint_name and list of question_no wise constraint allotment.
    Example:
        >>> from pprint import pprint
        >>> mark_distributions = {\
                "question": [5, 5, 10, 4, 6, 5, 5],\
                "unit": [8, 8, 8, 5, 11],\
                "difficulty": [13, 15, 12],\
                "cognitive": [12, 18, 10],\
            }
        >>> question_no = [1, 1, 2, 3, 3, 4, 4]
        >>> qpt = QPTGenerator(mark_distributions, question_no)
        >>> pprint(qpt.generate())
        {'cognitive': [2, 3, 2, 3, 3, 1, 3, 1, 1, 2],
         'difficulty': [3, 1, 2, 2, 1, 3, 3, 1, 2, 3],
         'question': [5, 5, 8, 2, 2, 1, 1, 6, 5, 5],
         'question_no': [1, 1, 2, 2, 3, 3, 3, 3, 4, 4],
         'unit': [4, 5, 1, 3, 2, 2, 3, 5, 2, 3]}
    """
    cdef QuestionPaper c_qpt

    def __cinit__(self, mark_of, question_no):
        self.c_qpt = QuestionPaper(mark_of, question_no)

    def generate(self):
        self.c_qpt.generate()
        return self.c_qpt.allot_by_question_no



# distutils: language = c++
from libcpp.string cimport string
from libcpp.unordered_map cimport unordered_map
from libcpp.vector cimport vector

cdef extern from "QPT.cpp":
    pass

cdef extern from "QPT.h" namespace "QPT":
    cdef cppclass QuestionPaper:
        QuestionPaper() except +
        QuestionPaper(unordered_map[string, vector[int]],
                      vector[int]) except +
        void generate()

        unordered_map[string, vector[int]] mark_of
        unordered_map[string, vector[int]] allot_by_question_no



import os

import pip

try:
    from Cython.Distutils.build_ext import new_build_ext
except ImportError:
    pip.main(['install', "cython"])
    from Cython.Distutils.build_ext import new_build_ext
from Cython.Build import cythonize

from setuptools import setup, find_packages, Extension
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

os.environ["CC"] = "g++"
os.environ["CXX"] = "g++"
extensions = [Extension("qpt_generator.qpt_generator",
                        sources=["qpt_generator/qpt_generator.pyx"],
                        language="c++",
                        extra_compile_args=["-std=c++11"],
                        extra_link_args=["-std=c++11"])]

setup(name="qpt_generator",
      version="0.1.5",
      description="Question Paper Template Generator",
      long_description=long_description,
      long_description_content_type='text/markdown',
      author="Niraj Kamdar",
      package_data={
          'qpt_generator': ["*.pxd", "*.pyx", "*.cpp", "*.h"]
      },
      packages=find_packages(),
      cmdclass={'build_ext': new_build_ext},
      ext_modules=cythonize(extensions, include_path=["qpt_generator"]),
      license='MIT',
      url='https://github.com/Niraj-Kamdar/qpt_generator',
      download_url='https://github.com/Niraj-Kamdar/qpt_generator/archive/master.zip',
      keywords=['question', 'paper', 'template', 'generator'],
      classifiers=[
          'Development Status :: 4 - Beta',
          # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          "Natural Language :: English",
          "Operating System :: OS Independent",
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
      ],
      )
